@@ -41,6 +41,7 @@ import javax.xml.validation.Schema;
 
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
+import org.w3c.dom.Node;
 
 import org.apache.cxf.Bus;
 import org.apache.cxf.binding.BindingFactoryManager;
@@ -48,6 +49,7 @@ import org.apache.cxf.catalog.CatalogWSDLLocator;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.common.xmlschema.SchemaCollection;
 import org.apache.cxf.helpers.CastUtils;
+import org.apache.cxf.helpers.DOMUtils;
 import org.apache.cxf.helpers.XMLUtils;
 import org.apache.cxf.service.model.BindingFaultInfo;
 import org.apache.cxf.service.model.BindingInfo;
@@ -68,7 +70,6 @@ import org.apache.ws.commons.schema.XmlSchemaElement;
 import org.easymock.classextension.EasyMock;
 import org.easymock.classextension.IMocksControl;
 import org.junit.Assert;
-import org.junit.Before;
 import org.junit.Test;
 
 public class WSDLServiceBuilderTest extends Assert {
@@ -78,6 +79,7 @@ public class WSDLServiceBuilderTest extends Assert {
     private static final String BARE_WSDL_PATH = "hello_world_bare.wsdl";
     private static final String IMPORT_WSDL_PATH = "hello_world_schema_import.wsdl";
     private static final String MULTIPORT_WSDL_PATH = "hello_world_multiporttype.wsdl";
+    private static final String NO_BODY_PARTS_WSDL_PATH = "no_body_parts.wsdl";
     
     private static final String EXTENSION_NAMESPACE = "http://cxf.apache.org/extension/ns";
     private static final QName EXTENSION_ATTR_BOOLEAN = new QName(EXTENSION_NAMESPACE, "booleanAttr");
@@ -99,8 +101,7 @@ public class WSDLServiceBuilderTest extends Assert {
 
     private DestinationFactoryManager destinationFactoryManager;
 
-    @Before
-    public void setUp() throws Exception {
+    public void setUpBasic() throws Exception {
         setUpWSDL(WSDL_PATH, 0);
     }
 
@@ -161,13 +162,14 @@ public class WSDLServiceBuilderTest extends Assert {
 
     @Test
     public void testServiceInfo() throws Exception {
+        setUpBasic();
         assertEquals("SOAPService", serviceInfo.getName().getLocalPart());
         assertEquals("http://apache.org/hello_world_soap_http", serviceInfo.getName().getNamespaceURI());
         assertEquals("http://apache.org/hello_world_soap_http", serviceInfo.getTargetNamespace());
         assertTrue(serviceInfo.getProperty(WSDLServiceBuilder.WSDL_DEFINITION) == def);
         assertTrue(serviceInfo.getProperty(WSDLServiceBuilder.WSDL_SERVICE) == service);
 
-        assertEquals("Incorrect number of endpoints", serviceInfo.getEndpoints().size(), 1);
+        assertEquals("Incorrect number of endpoints", 1, serviceInfo.getEndpoints().size());
         EndpointInfo ei = serviceInfo.getEndpoint(new QName("http://apache.org/hello_world_soap_http",
                 "SoapPort"));
         assertNotNull(ei);
@@ -178,14 +180,16 @@ public class WSDLServiceBuilderTest extends Assert {
 
     @Test
     public void testInterfaceInfo() throws Exception {
+        setUpBasic();
         assertEquals("Greeter", serviceInfo.getInterface().getName().getLocalPart());
         control.verify();
     }
 
     @Test
     public void testOperationInfo() throws Exception {
+        setUpBasic();
         QName name = new QName(serviceInfo.getName().getNamespaceURI(), "sayHi");
-        assertEquals(serviceInfo.getInterface().getOperations().size(), 4);
+        assertEquals(4, serviceInfo.getInterface().getOperations().size());
         OperationInfo sayHi = serviceInfo.getInterface().getOperation(
                 new QName(serviceInfo.getName().getNamespaceURI(), "sayHi"));
         assertNotNull(sayHi);
@@ -263,6 +267,7 @@ public class WSDLServiceBuilderTest extends Assert {
 
     @Test
     public void testBindingInfo() throws Exception {
+        setUpBasic();
         BindingInfo bindingInfo = null;
         assertEquals(1, serviceInfo.getBindings().size());
         bindingInfo = serviceInfo.getBindings().iterator().next();
@@ -275,6 +280,7 @@ public class WSDLServiceBuilderTest extends Assert {
 
     @Test
     public void testBindingOperationInfo() throws Exception {
+        setUpBasic();
         BindingInfo bindingInfo = null;
         bindingInfo = serviceInfo.getBindings().iterator().next();
         Collection<BindingOperationInfo> bindingOperationInfos = bindingInfo.getOperations();
@@ -306,6 +312,7 @@ public class WSDLServiceBuilderTest extends Assert {
 
     @Test
     public void testBindingMessageInfo() throws Exception {
+        setUpBasic();
         BindingInfo bindingInfo = null;
         bindingInfo = serviceInfo.getBindings().iterator().next();
 
@@ -363,10 +370,11 @@ public class WSDLServiceBuilderTest extends Assert {
     }
 
     @Test
-    public void testSchema() {
+    public void testSchema() throws Exception {
+        setUpBasic();
         SchemaCollection schemas = serviceInfo.getXmlSchemaCollection();
         assertNotNull(schemas);
-        assertEquals(serviceInfo.getSchemas().size(), 1);
+        assertEquals(1, serviceInfo.getSchemas().size());
         SchemaInfo schemaInfo = serviceInfo.getSchemas().iterator().next();
         assertNotNull(schemaInfo);
         assertEquals(schemaInfo.getNamespaceURI(), "http://apache.org/hello_world_soap_http/types");
@@ -378,6 +386,22 @@ public class WSDLServiceBuilderTest extends Assert {
         assertNotNull(schema);
         control.verify();
     }
+    
+    @Test
+    public void testNoBodyParts() throws Exception {
+        setUpWSDL(NO_BODY_PARTS_WSDL_PATH, 0);
+        QName messageName = new QName("urn:org:apache:cxf:no_body_parts/wsdl",
+                                      "operation1Request");
+        MessageInfo mi = serviceInfo.getMessage(messageName);
+        QName partName = new QName("urn:org:apache:cxf:no_body_parts/wsdl",
+                                   "mimeAttachment");
+        MessagePartInfo pi = mi.getMessagePart(partName);
+        QName typeName = 
+            new QName("http://www.w3.org/2001/XMLSchema",
+                      "base64Binary");
+        assertEquals(typeName, pi.getTypeQName());
+        assertNull(pi.getElementQName());
+    }
 
     @Test
     public void testBare() throws Exception {
@@ -404,18 +428,15 @@ public class WSDLServiceBuilderTest extends Assert {
         DocumentBuilder db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
         Document doc = db.parse(this.getClass().getResourceAsStream("./s1/s2/schema2.xsd"));
         Element schemaImport = null;
-        for (int i = 0; i < doc.getChildNodes().getLength(); i++) {
-            if (doc.getChildNodes().item(i) instanceof Element) {
-                Element schema = (Element) doc.getChildNodes().item(i);
-                for (int j = 0; j < schema.getChildNodes().getLength(); j++) {
-                    if (schema.getChildNodes().item(j) instanceof Element) {
-                        schemaImport = (Element) schema.getChildNodes().item(j);
-                        break;
-                    }
-                }
-                break;
+        
+        Node node = doc.getFirstChild();
+        while (node != null) {
+            if (node instanceof Element) {
+                schemaImport  = DOMUtils.getFirstElement(node);                
             }
+            node = node.getNextSibling();
         }
+        
         if (schemaImport == null) {
             fail("Can't find import element");
         }
