@@ -329,6 +329,8 @@ public class HTTPConduitURLEasyMockTest extends Assert {
         message.setExchange(exchange);
         exchange.isOneWay();
         EasyMock.expectLastCall().andReturn(true);
+        exchange.isSynchronous();
+        EasyMock.expectLastCall().andReturn(true);
     }
 
     private HTTPConduit setUpConduit(
@@ -349,8 +351,8 @@ public class HTTPConduitURLEasyMockTest extends Assert {
             connectionFactory.createConnection(
                                       EasyMock.eq(proxy), 
                                       EasyMock.eq(new URL(NOWHERE + "bar/foo")));
-            
             EasyMock.expectLastCall().andReturn(connection);
+
             connection.setDoOutput(true);
             EasyMock.expectLastCall();
             
@@ -505,7 +507,7 @@ public class HTTPConduitURLEasyMockTest extends Assert {
         wrappedOS.flush();
         wrappedOS.flush();
         wrappedOS.close();
-        
+
         assertNotNull("expected in message", inMessage);
         Map<?, ?> headerMap = (Map<?, ?>) inMessage.get(Message.PROTOCOL_HEADERS);
         assertEquals("unexpected response headers", headerMap.size(), 0);
@@ -560,6 +562,8 @@ public class HTTPConduitURLEasyMockTest extends Assert {
                                           EasyMock.eq("charset=utf8"));
             EasyMock.expectLastCall();
         }
+        connection.getRequestProperties();
+        EasyMock.expectLastCall().andReturn(new HashMap<String, List<String>>()).anyTimes();
         
         control.replay();
         
