@@ -23,11 +23,15 @@ import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.io.UnsupportedEncodingException;
 import java.util.Map;
+import java.util.UUID;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.atomic.AtomicLong;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
+import javax.jms.Destination;
 import javax.jms.JMSException;
 import javax.jms.MessageListener;
 import javax.jms.Session;
@@ -36,6 +40,7 @@ import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.message.Exchange;
 import org.apache.cxf.message.Message;
 import org.apache.cxf.message.MessageImpl;
+import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.transport.AbstractConduit;
 import org.apache.cxf.ws.addressing.EndpointReferenceType;
 import org.springframework.jms.core.JmsTemplate;
@@ -51,30 +56,35 @@ import org.springframework.jms.support.JmsUtils;
  */
 public class JMSConduit extends AbstractConduit implements JMSExchangeSender, MessageListener {
     static final Logger LOG = LogUtils.getL7dLogger(JMSConduit.class);
-
+    
+    private static final String CORRELATED = JMSConduit.class.getName() + ".correlated";
+    
+    private EndpointInfo endpointInfo;
     private JMSConfiguration jmsConfig;
-    private Map<String, Message> correlationMap;
-
+    private Map<String, Exchange> correlationMap;
     private DefaultMessageListenerContainer jmsListener;
+    private String conduitId;
+    private AtomicLong messageCount;
 
-    public JMSConduit(EndpointReferenceType target, JMSConfiguration jmsConfig) {
+    public JMSConduit(EndpointInfo endpointInfo, EndpointReferenceType target, JMSConfiguration jmsConfig) {
         super(target);
         this.jmsConfig = jmsConfig;
-        correlationMap = new ConcurrentHashMap<String, Message>();
+        this.endpointInfo = endpointInfo;
+        correlationMap = new ConcurrentHashMap<String, Exchange>();
+        conduitId = UUID.randomUUID().toString().replaceAll("-", "");
+        messageCount = new AtomicLong(0);
     }
-
+    
     /**
      * Prepare the message for send out. The message will be sent after the caller has written the payload to
      * the OutputStream of the message and calls the close method of the stream. In the JMS case the
      * JMSOutputStream will then call back the sendExchange method of this class. {@inheritDoc}
      */
     public void prepare(Message message) throws IOException {
-        if (jmsConfig.getTargetDestination() == null || jmsConfig.getConnectionFactory() == null) {
-            String name =  ".jms-conduit";
-            throw new RuntimeException("Insufficient configuration for Conduit. "
-                                       + "Did you configure a <jms:conduit name=\"" + name
-                                       + "\"> and set the jndiConnectionFactoryName ?");
-        }
+        String name =  endpointInfo.getName().toString() + ".jms-conduit";
+        org.apache.cxf.common.i18n.Message msg = 
+            new org.apache.cxf.common.i18n.Message("INSUFFICIENT_CONFIGURATION_CONDUIT", LOG, name);
+        jmsConfig.ensureProperlyConfigured(msg);
         boolean isTextPayload = JMSConstants.TEXT_MESSAGE_TYPE.equals(jmsConfig.getMessageType());
         JMSOutputStream out = new JMSOutputStream(this, message.getExchange(), isTextPayload);
         message.setContent(OutputStream.class, out);
@@ -95,35 +105,43 @@ public class JMSConduit extends AbstractConduit implements JMSExchangeSender, Me
             throw new RuntimeException("Exchange to be sent has no outMessage");
         }
 
-        JMSMessageHeadersType headers = (JMSMessageHeadersType)outMessage
+        final JMSMessageHeadersType headers = (JMSMessageHeadersType)outMessage
             .get(JMSConstants.JMS_CLIENT_REQUEST_HEADERS);
 
-        JmsTemplate jmsTemplate = JMSFactory.createJmsTemplate(jmsConfig, headers);
+        final JmsTemplate jmsTemplate = JMSFactory.createJmsTemplate(jmsConfig, headers);
         if (!exchange.isOneWay() && jmsListener == null) {
-            jmsListener = JMSFactory.createJmsListener(jmsConfig, this, jmsConfig.getReplyDestination());
+            jmsListener = JMSFactory.createJmsListener(jmsConfig, this, jmsConfig.getReplyDestination(), 
+                                                       conduitId);
         }
         
         final javax.jms.Destination replyTo = exchange.isOneWay() ? null : jmsListener.getDestination();
-
-        final String correlationId = (headers != null && headers.isSetJMSCorrelationID()) ? headers
-            .getJMSCorrelationID() : JMSUtils.generateCorrelationId();
-        // String selector = "JMSCorrelationID = '" + correlationId + "'";
-        Message inMessage = null;
-        if (!exchange.isOneWay()) {
-            inMessage = new MessageImpl();
-            correlationMap.put(correlationId, inMessage);
-        }
-        jmsTemplate.send(jmsConfig.getTargetDestination(), new MessageCreator() {
+        
+        final String correlationId = (headers != null && headers.isSetJMSCorrelationID()) 
+            ? headers.getJMSCorrelationID() 
+            : JMSUtils.createCorrelationId(jmsConfig.getConduitSelectorPrefix() + conduitId, 
+                                           messageCount.incrementAndGet());
+        
+        MessageCreator messageCreator = new MessageCreator() {
             public javax.jms.Message createMessage(Session session) throws JMSException {
                 String messageType = jmsConfig.getMessageType();
                 final javax.jms.Message jmsMessage;
+                Destination replyToDestination = replyTo;
+                if (exchange.isOneWay() && !jmsConfig.isEnforceSpec()) {
+                    final String contextReplyToName = (headers != null) ? headers.getJMSReplyTo() : null;
+                    if (contextReplyToName != null) {
+                        replyToDestination = 
+                            JMSFactory.resolveOrCreateDestination(jmsTemplate, 
+                                                                  contextReplyToName, 
+                                                                  jmsConfig.isPubSubDomain());
+                    }
+                }
                 jmsMessage = JMSUtils.buildJMSMessageFromCXFMessage(outMessage, request, messageType,
-                                                                    session, replyTo,
+                                                                    session, replyToDestination,
                                                                     correlationId);
                 LOG.log(Level.FINE, "client sending request: ", jmsMessage);
                 return jmsMessage;
             }
-        });
+        };
 
         /**
          * If the message is not oneWay we will expect to receive a reply on the listener. To receive this
@@ -131,22 +149,28 @@ public class JMSConduit extends AbstractConduit implements JMSExchangeSender, Me
          * fill to Message and notify this thread
          */
         if (!exchange.isOneWay()) {
-            synchronized (inMessage) {
-                try {
-                    inMessage.wait(jmsTemplate.getReceiveTimeout());
-                } catch (InterruptedException e) {
-                    throw new RuntimeException(e);
-                }
-                correlationMap.remove(correlationId);
-                if (inMessage.getContent(InputStream.class) == null) {
-                    throw new RuntimeException("Timeout receiving message with correlationId "
-                                               + correlationId);
+            synchronized (exchange) {
+                correlationMap.put(correlationId, exchange);
+                jmsTemplate.send(jmsConfig.getTargetDestination(), messageCreator);
+                
+                if (exchange.isSynchronous()) {
+                    try {
+                        exchange.wait(jmsTemplate.getReceiveTimeout());
+                    } catch (InterruptedException e) {
+                        correlationMap.remove(correlationId);
+                        throw new RuntimeException(e);
+                    }
+                    correlationMap.remove(correlationId);
+                    if (exchange.get(CORRELATED) == null) {
+                        throw new RuntimeException("Timeout receiving message with correlationId "
+                                                   + correlationId);
+                    }
+                    
+                    
                 }
             }
-            exchange.setInMessage(inMessage);
-            if (incomingObserver != null) {
-                incomingObserver.onMessage(inMessage);
-            }
+        } else {
+            jmsTemplate.send(jmsConfig.getTargetDestination(), messageCreator);
         }
     }
 
@@ -162,20 +186,36 @@ public class JMSConduit extends AbstractConduit implements JMSExchangeSender, Me
         } catch (JMSException e) {
             throw JmsUtils.convertJmsAccessException(e);
         }
-        Message inMessage = correlationMap.get(correlationId);
-        if (inMessage == null) {
+
+        Exchange exchange = correlationMap.remove(correlationId);
+        if (exchange == null) {
             LOG.log(Level.WARNING, "Could not correlate message with correlationId " + correlationId);
+            return;
         }
+        Message inMessage = new MessageImpl();
+        exchange.setInMessage(inMessage);
         LOG.log(Level.FINE, "client received reply: ", jmsMessage);
-        JMSUtils.populateIncomingContext(jmsMessage, inMessage, JMSConstants.JMS_CLIENT_RESPONSE_HEADERS);
-        byte[] response = JMSUtils.retrievePayload(jmsMessage);
-        LOG.log(Level.FINE, "The Response Message payload is : [" + response + "]");
-        inMessage.setContent(InputStream.class, new ByteArrayInputStream(response));
+        try {
+            JMSUtils.populateIncomingContext(jmsMessage, inMessage, JMSConstants.JMS_CLIENT_RESPONSE_HEADERS);
+        
+            byte[] response = JMSUtils.retrievePayload(jmsMessage, (String)inMessage.get(Message.ENCODING));
+            LOG.log(Level.FINE, "The Response Message payload is : [" + response + "]");
+            inMessage.setContent(InputStream.class, new ByteArrayInputStream(response));
 
-        synchronized (inMessage) {
-            inMessage.notifyAll();
+            if (exchange.isSynchronous()) {
+                synchronized (exchange) {
+                    exchange.put(CORRELATED, Boolean.TRUE);
+                    exchange.notifyAll();
+                }
+            }
+        
+            //REVISIT: put on a workqueue?
+            if (incomingObserver != null) {
+                incomingObserver.onMessage(exchange.getInMessage());
+            }
+        } catch (UnsupportedEncodingException ex) {
+            getLogger().log(Level.WARNING, "can't get the right encoding information " + ex);
         }
-
     }
 
     public void close() {
