@@ -19,34 +19,57 @@
 package org.apache.cxf.systest.jaxws;
 
 import java.util.Arrays;
+import java.util.LinkedHashSet;
 import java.util.List;
+import java.util.Set;
 import java.util.Vector;
 
+import javax.annotation.Resource;
 import javax.jws.WebService;
 import javax.xml.ws.Holder;
+import javax.xml.ws.WebServiceContext;
 
+import org.apache.cxf.feature.Features;
 import org.apache.cxf.message.Exchange;
+import org.apache.cxf.systest.jaxws.DocLitWrappedCodeFirstService.Foo;
+import org.apache.cxf.systest.jaxws.types.Bar;
+import org.apache.cxf.systest.jaxws.types.BarImpl;
 
 @WebService(endpointInterface = "org.apache.cxf.systest.jaxws.DocLitWrappedCodeFirstService",
             serviceName = "DocLitWrappedCodeFirstService",
             portName = "DocLitWrappedCodeFirstServicePort",
             targetNamespace = "http://cxf.apache.org/systest/jaxws/DocLitWrappedCodeFirstService")
+//@Features(features = { "org.apache.cxf.feature.FastInfosetFeature" })
+@Features(features = { "org.apache.cxf.transport.http.gzip.GZIPFeature", 
+                       "org.apache.cxf.feature.FastInfosetFeature" })
 public class DocLitWrappedCodeFirstServiceImpl implements DocLitWrappedCodeFirstService {
     public static final String DATA[] = new String[] {"string1", "string2", "string3"};
     
+    @Resource
+    WebServiceContext context;
+    
     public int thisShouldNotBeInTheWSDL(int i) {
         return i;
     }
     
     public String[] arrayOutput() {
+        if (context == null) {
+            throw new RuntimeException("No CONTEXT!!!");
+        }
         return DATA;
     }
 
     public Vector<String> listOutput() {
+        if (context == null) {
+            throw new RuntimeException("No CONTEXT!!!");
+        }
         return new Vector<String>(Arrays.asList(DATA));
     }
 
     public String arrayInput(String[] inputs) {
+        if (context == null) {
+            throw new RuntimeException("No CONTEXT!!!");
+        }
         StringBuffer buf = new StringBuffer();
         for (String s : inputs) {
             buf.append(s);
@@ -55,11 +78,17 @@ public class DocLitWrappedCodeFirstServiceImpl implements DocLitWrappedCodeFirst
     }
     
     public int[] echoIntArray(int[] ar, Exchange ex) {
+        if (context == null) {
+            throw new RuntimeException("No CONTEXT!!!");
+        }
         return ar;
     }
 
 
     public String listInput(List<String> inputs) {
+        if (context == null) {
+            throw new RuntimeException("No CONTEXT!!!");
+        }
         StringBuffer buf = new StringBuffer();
         if (inputs != null) {
             for (String s : inputs) {
@@ -70,6 +99,9 @@ public class DocLitWrappedCodeFirstServiceImpl implements DocLitWrappedCodeFirst
     }
     
     public String multiListInput(List<String> inputs1, List<String> inputs2, String x, int y) {
+        if (context == null) {
+            throw new RuntimeException("No CONTEXT!!!");
+        }
         StringBuffer buf = new StringBuffer();
         for (String s : inputs1) {
             buf.append(s);
@@ -106,6 +138,10 @@ public class DocLitWrappedCodeFirstServiceImpl implements DocLitWrappedCodeFirst
         b.setName("b");
         return Arrays.asList(a, b);
     }
+    
+    public Set<Foo> getFooSet() {
+        return new LinkedHashSet<Foo>(listObjectOutput());
+    }
 
     public List<Foo[]> listObjectArrayOutput() {
         Foo a = new Foo();
@@ -137,6 +173,7 @@ public class DocLitWrappedCodeFirstServiceImpl implements DocLitWrappedCodeFirst
             ComplexException.MyBean bean = new ComplexException.MyBean();
             bean.setName("Marco");
             ex.setBeans(new ComplexException.MyBean[] {bean});
+            ex.setInts(new int[] {1, 2, 3});
             throw ex;
         }
         default:
@@ -152,8 +189,20 @@ public class DocLitWrappedCodeFirstServiceImpl implements DocLitWrappedCodeFirst
         return i;
     }
 
+    public Bar createBar(String val) {
+        return new BarImpl(val);
+    }
 
     public boolean listObjectIn(Holder<List<Foo[]>> foos) {
         return false;
     }
+
+    public void doOneWay() {
+        try {
+            Thread.sleep(2000);
+        } catch (InterruptedException e) {
+            // TODO Auto-generated catch block
+            e.printStackTrace();
+        }
+    }
 }
